//******************************************************************************
//* SBSS version 5.x
//* Module revision: 2014.05.28
//* www.rusbss.ru
//******************************************************************************

{$I ssReader.inc}

unit ssReaderIntf;

interface

uses
  Variants, Classes, Graphics,
{$IFDEF USEADODATASET}
  ADODB,
{$ENDIF}
{$IFDEF USECLIENTDATASET}
  DbClient,
{$ENDIF}
  Dialogs;

const
  MaxHostNum = 4;
  MaxProxyNum = 2;

type
  TAddLogMethodW = procedure(const S: WideString;
                             AInfoType: TMsgDlgType;
                             ACustomColor: TColor = clBlack) of object;
  Str40 = string[40];
  Str128 = string[128];
  Str255 = string[255];

  TDataHost = packed record
    ComputerName: Str128;
    Port: Integer;
  end;

  TArrayHost = array[0..MaxHostNum] of TDataHost;

  TDataCommonRDM = packed record
    ConnectionName: Str128;
    TmpDir: Str255;
    RDMServerName: Str40;
    RDMServerGUID: Str40;
    UseTransaction: Boolean;
    ConnectionType: Integer; // 0-TSocketConnection 1-TWebConnection
  end;

  TDataSocket = packed record
    InterceptGUID: Str40;
    Hosts: TArrayHost;
  end;

  TDataWeb = packed record
    Proxy: Str128;
    ProxyByPass: Str128;
    UserName: Str40;
    Password: Str40;
    Hosts: TArrayHost;
  end;

  TDataRas = packed record
    PhoneBook: Str255;
    EntryName: Str128;
    UserName: Str40;
    Password: Str40;
    HangupBeforeDial: Boolean;
    UseRas: Boolean;
  end;

  TDataTuneMidas1 = packed record
    Common: TDataCommonRDM;
    Socket: TDataSocket;
    Web: TDataWeb;
    Ras: TDataRas;
    Use: Boolean;
  end;

  TSSReaderDataTunes = array[0..MaxProxyNum] of TDataTuneMidas1;

function ssReader_GetVersion: Double; StdCall;
//   

procedure ssReader_SetLanguage(ALng: Integer); StdCall;
//   
// English = 1033 ($409)
// Russian = 1049 ($419)

function ssReader_InterceptCheck: Boolean; StdCall;
//      mssdiz.dll
//  ,  C:\SBSS\mssdiz.dll
//  

procedure ssReader_Init(AAddLogMethod: TAddLogMethodW); StdCall;
//       ssServer

procedure ssReader_Done; StdCall;
//       ssServer

procedure ssReader_ClearTunes; StdCall;
//  

procedure ssReader_SetTunes(const AData: TSSReaderDataTunes); StdCall;
//  

procedure ssReader_GetTunes(var AData: TSSReaderDataTunes); StdCall;
//   

procedure ssReader_ReadTunes; StdCall;
//     ssReader.opt
//   ,  

procedure ssReader_WriteTunes; StdCall;
//     ssReader.opt
//   ,  

procedure ssReader_DialogTunes(const ACaption: WideString; ADoRasHangup: Boolean); StdCall;
//     
//    ssServer
//   
//   OK,     ssReader.opt
//   ,  

function ssReader_OpenConnection(const AGroupName: WideString): WideString; StdCall;
//    ssServer
//         AGroupName
//  SQL-   SSDATAREADER  
//  OK,       
//   OK,    

procedure ssReader_CloseConnection(ADoRasHangup: Boolean); StdCall;
//     
//    ssServer

function ssReader_GetDataADODataSet(const ACommandText: WideString;
  AFormat: Integer;
  var AData: OleVariant): Boolean; StdCall;
//   ACommandText
//     AData
//  AFormat = 0 AData    ADO Stream
//  AFormat = 1 AData   ADO XML
//     ssReader_LoadADODataSet

function ssReader_GetDataClientDataSet(const ACommandText: WideString;
  AFormat: Integer;
  var AData: OleVariant): Boolean; StdCall;
//   ACommandText
//     AData
//  AFormat = 0 AData    ClientDataSet
//  AFormat = 1 AData   ClientDataSet XML
//     ssReader_LoadClentDataSet

{$IFDEF USEADODATASET}
function ssReader_LoadADODataSet(ADataSet: TCustomADODataSet;
  const ACommandText: WideString;
  AFormat: Integer): Boolean;
//  brief-case ADataSet
//  AFormat = 0   
//  AFormat = 1   XML
{$ENDIF}

{$IFDEF USECLIENTDATASET}
function ssReader_LoadClentDataSet(ADataSet: TClientDataSet;
  const ACommandText: WideString;
  AFormat: Integer): Boolean;
//  brief-case ADataSet
//  AFormat = 0   
//  AFormat = 1   XML
{$ENDIF}

implementation

{$IFDEF USEADODATASET}
uses
{$IF CompilerVersion < 21}
  ActiveX, ComObj,
  ADOInt,
  ADOInt28;
{$ELSE}
  ADOInt;
{$IFEND}

type
  TCustomADODataSetPatch = class(TCustomADODataSet)
  public
    property CommandText;
    property CommandType;
  end;

procedure ADODataSetFromStream(ADataSet: TCustomADODataSet;
  AStream: OleVariant);
var
  FRecordsetObject: ADOINT._Recordset;
begin
  ADataSet.Close;
  if (ADataSet is TADOQuery) then
    (ADataSet as TADOQuery).SQL.Clear
  else
    TCustomADODataSetPatch(ADataSet).CommandText := '';
  TCustomADODataSetPatch(ADataSet).CommandType := ADODB.cmdFile;
  ADataSet.LockType := ADODB.ltBatchOptimistic;

  FRecordsetObject := ADOINT.CoRecordset.Create;
  try
    FRecordsetObject.CursorLocation := ADOINT.adUseClient;
    FRecordsetObject.Open(AStream, VARIANTS.EmptyParam, adOpenUnspecified, adLockUnspecified, 0);
    ADataSet.Recordset := FRecordsetObject;
    ADataSet.Open;
  except
    FRecordsetObject := nil;
    raise;
  end;
end;

procedure ADODataSetFromRawData(ADataSet: TCustomADODataSet;
  AData: OleVariant;
  AFormat: TPersistFormat = pfADTG);
var
  ADOStream: _Stream;
begin
  ADOStream := CoStream.Create;
  try
    if AFormat = pfADTG then
      ADOStream.Type_ := adTypeBinary
    else
      ADOStream.Type_ := adTypeText;

    ADOStream.Open(
      VARIANTS.EmptyParam,     // Source: OleVariant;
      ADOINT.adModeUnknown,    // Mode: ConnectModeEnum;
      adOpenStreamUnspecified, // Options: StreamOpenOptionsEnum;
      '',                      // const UserName: WideString;
      '');                     // const Password: WideString);

    if AFormat = pfADTG then
      ADOStream.Write(AData)
    else
      ADOStream.WriteText(AData, stWriteLine);

    ADOStream.Position := 0;
    ADODataSetFromStream(ADataSet, ADOStream);
  finally
    ADOStream := nil;
  end;
end;

function ssReader_LoadADODataSet(ADataSet: TCustomADODataSet;
  const ACommandText: WideString;
  AFormat: Integer): Boolean;

  procedure ADODataSetSetData(ADataSet: TCustomADODataSet;
    AFormat: Integer;
    var AData: OleVariant);
  var
    Format: ADODB.TPersistFormat;
  begin
    case AFormat of
      0: Format := pfADTG;
    else
      Format := pfXML;
    end;
    ADODataSetFromRawData(ADataSet, AData, Format);
    AData := VARIANTS.Null;
  end;

var
  Data: OleVariant;
begin
  Result := False;
  if ssReader_GetDataADODataSet(ACommandText, AFormat, Data) then
  begin
    ADODataSetSetData(ADataSet, AFormat, Data);
    Result := True;
  end;
end;
{$ENDIF}

{$IFDEF USECLIENTDATASET}
function ssReader_LoadClentDataSet(ADataSet: TClientDataSet;
  const ACommandText: WideString;
  AFormat: Integer): Boolean;

  procedure ClientDataSetSetData(ADataSet: TClientDataSet;
    AFormat: Integer;
    var AData: OleVariant);
  begin
    case AFormat of
      0: ADataSet.Data := AData;
    else
      ADataSet.XMLData := AData;
    end;
    AData := VARIANTS.Null;
  end;

var
  Data: OleVariant;
begin
  Result := False;
  if ssReader_GetDataClientDataSet(ACommandText, AFormat, Data) then
  begin
    ClientDataSetSetData(ADataSet, AFormat, Data);
    Result := True;
  end;
end;
{$ENDIF}

//------------------------------------------------------------------------------

const
  ssReaderDLL = 'ssReader.dll';

function ssReader_GetVersion;
external ssReaderDLL name 'ssReader_GetVersion';

procedure ssReader_SetLanguage;
external ssReaderDLL name 'ssReader_SetLanguage';

function ssReader_InterceptCheck;
external ssReaderDLL name 'ssReader_InterceptCheck';

procedure ssReader_Init;
external ssReaderDLL name 'ssReader_Init';

procedure ssReader_Done;
external ssReaderDLL name 'ssReader_Done';

procedure ssReader_ClearTunes;
external ssReaderDLL name 'ssReader_ClearTunes';

procedure ssReader_SetTunes;
external ssReaderDLL name 'ssReader_SetTunes';

procedure ssReader_GetTunes;
external ssReaderDLL name 'ssReader_GetTunes';

procedure ssReader_ReadTunes;
external ssReaderDLL name 'ssReader_ReadTunes';

procedure ssReader_WriteTunes;
external ssReaderDLL name 'ssReader_WriteTunes';

procedure ssReader_DialogTunes;
external ssReaderDLL name 'ssReader_DialogTunes';

function ssReader_OpenConnection;
external ssReaderDLL name 'ssReader_OpenConnection';

procedure ssReader_CloseConnection;
external ssReaderDLL name 'ssReader_CloseConnection';

function ssReader_GetDataADODataSet;
external ssReaderDLL name 'ssReader_GetDataADODataSet';

function ssReader_GetDataClientDataSet;
external ssReaderDLL name 'ssReader_GetDataClientDataSet';

end.
