{$I ssReader.inc}

unit ssReaderMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, DB, Grids, DBGrids, ComCtrls,
{$IFDEF USEADODATASET}
  ADODB,
{$ENDIF}
{$IFDEF USECLIENTDATASET}
  DbClient,
{$ENDIF}
  ssReaderIntf;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    Memo1: TMemo;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    Button5: TButton;
    DBGrid1: TDBGrid;
    Splitter1: TSplitter;
    RichEdit1: TRichEdit;
    DataSource1: TDataSource;
    GroupBox1: TGroupBox;
    RadioButton1: TRadioButton;
    RadioButton2: TRadioButton;
    CheckBox1: TCheckBox;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
  private
{$IFDEF USEADODATASET}
    FADOQuery: ADODB.TADOQuery;
{$ENDIF}
{$IFDEF USECLIENTDATASET}
    FClientDataSet: DBCLIENT.TClientDataSet;
{$ENDIF}
    FCommandText: string;
    FSaveCursor: TCursor;
    procedure DisableControls;
    procedure EnableControls;
  public
    procedure AddLog(const S: WideString; AInfoType: TMsgDlgType; ACustomColor: TColor = clBlack);

    procedure OpenConnection;
    procedure CloseConnection;

    procedure OpenQuery1;
    procedure CloseQuery1;
  end;

var
  Form1: TForm1;

function CheckTunes: Boolean;

implementation

{$R *.dfm}

var
  GGroupName: string;

function CheckTunes: Boolean;
begin
  Result := False;

  SYSUTILS.DateSeparator := '.';
  SYSUTILS.ShortDateFormat := 'dd.MM.yyyy';

  if ssReader_GetVersion < 5.3 then
  begin
    DIALOGS.ShowMessage(' ssReader.dll   >= 5.3');
    Exit;
  end;

  GGroupName := ParamStr(1);
  if GGroupName = '' then
  repeat
    if not DIALOGS.InputQuery('', '', GGroupName) then
    begin
      Application.Terminate;
      Exit;
    end;
    GGroupName := SYSUTILS.Trim(GGroupName);
  until GGroupName <> '';

  ssReader_ClearTunes;
  ssReader_ReadTunes;
  if ssReader_InterceptCheck then;

  Result := True;
end;

//--- TForm1 -----------------------------------------------------------

procedure TForm1.FormCreate(Sender: TObject);
var
  Version: Double;
  S: ShortString;
begin
  Left := 8;
  Top := 8;

{$IFDEF USEADODATASET}
  FADOQuery := TADOQuery.Create(SELF);
{$ELSE}
  RadioButton1.Enabled := False;
{$ENDIF}

{$IFDEF USECLIENTDATASET}
  FClientDataSet := TClientDataSet.Create(SELF);
{$ELSE}
  RadioButton2.Enabled := False;
{$ENDIF}

  Label1.Caption := '';

  Memo1.Lines.Clear;
  Memo1.Lines.Text := 'select GetDate()';

  RichEdit1.Lines.Clear;

  Version := ssReader_GetVersion;
  SYSTEM.Str(Version:3:1, S);
  Caption := 'ssReader ' + string(S);

  ssReader_Init(AddLog);
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  ssReader_Done;
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_F5) and (Shift = []) then
    Button4Click(Sender);
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  ssReader_DialogTunes('', False);
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  OpenConnection;
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
  CloseConnection;
end;

procedure TForm1.Button4Click(Sender: TObject);
begin
  OpenQuery1;
end;

procedure TForm1.Button5Click(Sender: TObject);
begin
  CloseQuery1;
end;

procedure TForm1.DisableControls;
begin
  FSaveCursor := Screen.Cursor;
  Screen.Cursor := crHourGlass;
  Button1.Enabled := False;
  Button2.Enabled := False;
  Button3.Enabled := False;
  Button4.Enabled := False;
  Button5.Enabled := False;
end;

procedure TForm1.EnableControls;
begin
  Button1.Enabled := True;
  Button2.Enabled := True;
  Button3.Enabled := True;
  Button4.Enabled := True;
  Button5.Enabled := True;
  Screen.Cursor := FSaveCursor;
end;

procedure TForm1.AddLog(const S: WideString; AInfoType: TMsgDlgType; ACustomColor: TColor = clBlack);
begin
  case AInfoType of
    mtInformation:
      begin
        RichEdit1.SelAttributes.Color := clBlack;
        RichEdit1.Lines.Add('I. ' + S);
      end;
    mtWarning:
      begin
        RichEdit1.SelAttributes.Color := clBlue;
        RichEdit1.Lines.Add('W. ' + S);
      end;
    mtError:
      begin
        RichEdit1.SelAttributes.Color := clRed;
        RichEdit1.Lines.Add('E. ' + S);
      end;
  else
    RichEdit1.SelAttributes.Color := ACustomColor;
    RichEdit1.Lines.Add(S);
  end;
  RichEdit1.Invalidate;
  RichEdit1.SetFocus;
  Application.ProcessMessages;
end;

procedure TForm1.OpenConnection;
var
  Info: string;
begin
  DisableControls;
  try
    AddLog('Open Connection', mtWarning);
    Info := ssReader_OpenConnection(GGroupName);
    if Info <> '' then // Ok
      AddLog(Info, mtInformation);
  finally
    EnableControls;
  end;
end;

procedure TForm1.CloseConnection;
begin
  DisableControls;
  try
    ssReader_CloseConnection(False);
    AddLog('Close Connection', mtWarning);
  finally
    EnableControls;
  end;
end;

procedure TForm1.OpenQuery1;
var
  T1, T2: TDateTime;
begin
  CloseQuery1;

  FCommandText := Memo1.Lines.Text;
  if SYSUTILS.Trim(FCommandText) = '' then Exit;

  DataSource1.DataSet := nil;

  DisableControls;
  try
{$IFDEF USEADODATASET}
    if RadioButton1.Checked then
    begin
      AddLog('Open ADOQuery', mtWarning);
      T1 := Now;
      if ssReader_LoadADODataSet(FADOQuery, FCommandText, Integer(CheckBox1.Checked)) then
      begin
        T2 := Now;
        DataSource1.DataSet := FADOQuery;
        Label1.Caption := TimeToStr(T2 - T1) + ' [Records ' + IntToStr(FADOQuery.RecordCount) + ']';
      end;
    end;
{$ENDIF}
{$IFDEF USECLIENTDATASET}
    if RadioButton2.Checked then
    begin
      AddLog('Open ClientDataSet', mtWarning);
      T1 := Now;
      if ssReader_LoadClentDataSet(FClientDataSet, FCommandText, Integer(CheckBox1.Checked)) then
      begin
        T2 := Now;
        DataSource1.DataSet := FClientDataSet;
        Label1.Caption := TimeToStr(T2 - T1) + ' [Records ' + IntToStr(FClientDataSet.RecordCount) + ']';
      end;
    end;
{$ENDIF}
  finally
    EnableControls;
  end;
end;

procedure TForm1.CloseQuery1;
begin
{$IFDEF USEADODATASET}
  FADOQuery.Close;
{$ENDIF}
{$IFDEF USECLIENTDATASET}
  FClientDataSet.Close;
{$ENDIF}
  AddLog('Close Query', mtWarning);
  Label1.Caption := '';
end;

end.
